<?php
namespace nur\sery\wip\php\access;

use ReflectionClass;
use ReflectionException;
use ReflectionProperty;

class PropertyAccess extends AbstractAccess {
  function __construct(object $object, $name, ?array $params=null) {
    parent::__construct($params);
    $this->object = $object;
    $this->name = $name;
    $class = new ReflectionClass($object);
    try {
      $property = $class->getProperty($this->name);
      $property->setAccessible(true);
    } catch (ReflectionException $e) {
      $property = null;
    }
    $this->property = $property;
    $this->allowNull = $params["allow_null"] ?? true;
    $this->allowFalse = $params["allow_false"] ?? false;
  }

  protected object $object;

  protected string $name;

  protected ?ReflectionProperty $property;

  protected bool $allowNull;

  protected bool $allowFalse;

  function reset(object $object): self {
    $this->object = $object;
    return $this;
  }

  function exists(): bool {
    return $this->property !== null
      || property_exists($this->object, $this->name);
  }

  protected function _get($default=null) {
    $name = $this->name;
    $property = $this->property;
    if ($property !== null) {
      return $property->getValue($this->object);
    } elseif (property_exists($this->object, $name)) {
      return $this->object->$name;
    } else {
      return $default;
    }
  }

  function available(): bool {
    if (!$this->exists()) return false;
    $value = $this->_get();
    if ($value === "") return $this->allowEmpty;
    if ($value === null) return $this->allowNull;
    if ($value === false) return $this->allowFalse;
    return true;
  }

  function get($default=null) {
    if (!$this->exists()) return $default;
    $value = $this->_get();
    if ($value === "" && !$this->allowEmpty) return $default;
    if ($value === null && !$this->allowNull) return $default;
    if ($value === false && !$this->allowFalse) return $default;
    return $value;
  }

  protected function _set($value): void {
    $name = $this->name;
    $property = $this->property;
    if ($property !== null) {
      $property->setValue($this->object, $value);
    } else {
      $this->object->$name = $value;
    }
  }

  function set($value): void {
    $this->_set($value);
  }

  function del(): void {
    $this->_set(null);
  }

  function addKey($key): IAccess {
    return new ChainAccess($this, $key);
  }
}
