<?php
namespace nur\sery\wip\schema\input;

use nur\sery\wip\php\access\KeyAccess;
use nur\sery\wip\php\access\PropertyAccess;

/**
 * Class Input: accès à une valeur
 *
 * cette implémentation lit depuis et écrit dans une référence
 */
class Input implements IInput {
  const ALLOW_EMPTY = true;

  function __construct(&$dest=null, ?array $params=null) {
    if (is_object($dest)) {
      $this->access = new PropertyAccess($dest, null, [
        "allow_empty" => $params["allow_empty"] ?? static::ALLOW_EMPTY,
        "allow_null" => true,
      ]);
    } else {
      $this->access = new KeyAccess($dest, null, [
        "allow_empty" => $params["allow_empty"] ?? static::ALLOW_EMPTY,
        "allow_null" => true,
      ]);
    }
  }

  protected KeyAccess $access;

  /** tester si la valeur existe sans tenir compte de $allowEmpty */
  function isPresent($key=null): bool {
    return $this->access->resetKey($key)->exists();
  }

  /** tester si la valeur est disponible en tenant compte de $allowEmpty */
  function isAvailable($key=null): bool {
    return $this->access->resetKey($key)->available();
  }

  function get($key=null) {
    return $this->access->resetKey($key)->get();
  }

  function set($value, $key=null): void {
    $this->access->resetKey($key)->set($value);
  }

  function unset($key=null): void {
    $this->access->resetKey($key)->del();
  }

  function addKey($key): IInput {
    return new ChainInput($this, $key);
  }
}
